/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogLinearScale.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"
#include "GuiDialogOAC.h"
#include "MathTrig.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogLinearScale::TFDialogLinearScale (HeightField *HF, HeightFieldDraw *HFD)
	        : TFPreviewDialog (HF, HFD, "Terraform Linear Scale Dialog", 
					"Parameters", 6, 8),
		  d_vbList (TRUE, 0),
		  d_hbCX (TRUE, 5),
		  d_hbCY (TRUE, 5),
		  d_hbScaleFactor (TRUE, 5),
		  d_hbMinDist (TRUE, 5),
		  d_hbMaxDist (TRUE, 5),
		  d_hbSmoothDist (TRUE, 5),
		  d_hbInvert (TRUE, 5),
		  d_hbFullDist (TRUE, 5),
		  d_optionsFrame ("Linear Scale Options")
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogLinearScale\n");

	d_lblCX = new Gtk_Label ("Center X"),
	d_lblCY = new Gtk_Label ("Center Y"),
	d_lblScaleFactor = new Gtk_Label ("Scale Factor"),
	d_lblMinDist = new Gtk_Label ("Scale Start Distance"),
	d_lblMaxDist = new Gtk_Label ("Scale End Distance"),
	d_lblSmoothDist = new Gtk_Label ("Smoothing Factor"),
	d_lblInvert = new Gtk_Label ("Invert Scale Factor"),
	d_lblFullDist = new Gtk_Label ("Scale against Max Distance"),

	d_adjCX = new Gtk_Adjustment (0.5, 0, 1, 0.01),
	d_adjCY = new Gtk_Adjustment (0.5, 0, 1, 0.01),
	d_adjScaleFactor = new Gtk_Adjustment (1.5, 1, 5, 0.01),
	d_adjMinDist = new Gtk_Adjustment (0.33, 0, 1, 0.01),
	d_adjMaxDist = new Gtk_Adjustment (0.66, 0, 1, 0.01),
	d_adjSmoothDist = new Gtk_Adjustment (0.1, 0, 1, 0.01),

	d_hsCX = new Gtk_HScale (*d_adjCX),
	d_hsCY = new Gtk_HScale (*d_adjCY),
	d_hsScaleFactor = new Gtk_HScale (*d_adjScaleFactor),
	d_hsMinDist = new Gtk_HScale (*d_adjMinDist),
	d_hsMaxDist = new Gtk_HScale (*d_adjMaxDist),
	d_hsSmoothDist = new Gtk_HScale (*d_adjSmoothDist),

	buildDialogWindow ();
	this->setHFobjs (HF, HFD);
	this->iterateEvents ();
	updatePreviewCallback ();

	connect_to_method (d_adjCX->value_changed, this, &TFDialogLinearScale::updatePreviewCallback);
	connect_to_method (d_adjCY->value_changed, this, &TFDialogLinearScale::updatePreviewCallback);
	connect_to_method (d_adjMinDist->value_changed, this, &TFDialogLinearScale::updatePreviewCallback);
	connect_to_method (d_adjMaxDist->value_changed, this, &TFDialogLinearScale::updatePreviewCallback);
	connect_to_method (d_adjScaleFactor->value_changed, this, &TFDialogLinearScale::updatePreviewCallback);
	connect_to_method (d_adjSmoothDist->value_changed, this, &TFDialogLinearScale::updatePreviewCallback);
	connect_to_method (d_btnInvert.clicked, this, &TFDialogLinearScale::updatePreviewCallback);
	connect_to_method (d_btnFullDist.clicked, this, &TFDialogLinearScale::updatePreviewCallback);
}


/*
 *  destructor: clean up 
 */
TFDialogLinearScale::~TFDialogLinearScale ()
{
	delete d_lblCX;
	delete d_lblCY;
	delete d_lblScaleFactor;
	delete d_lblMinDist;
	delete d_lblMaxDist;
	delete d_lblSmoothDist;
	delete d_lblInvert;
	delete d_lblFullDist;

	delete d_adjCX;
	delete d_adjCY;
	delete d_adjScaleFactor;
	delete d_adjMinDist;
	delete d_adjMaxDist;
	delete d_adjSmoothDist;

	delete d_hsCX;
	delete d_hsCY;
	delete d_hsScaleFactor;
	delete d_hsMinDist;
	delete d_hsMaxDist;
	delete d_hsSmoothDist;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogLinearScale\n");
}


/*
 *  insertOptions: fill the dialog's VBox
 */
void TFDialogLinearScale::insertOptions ()
{
	d_optionsFrame.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_optionsFrame.set_border_width (5);

	d_hbCX.pack_start (*d_lblCX, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblCX->show ();
	d_hsCX->set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsCX->set_digits (2);
	d_hsCX->set_draw_value (TRUE);
	d_hbCX.pack_end (*d_hsCX, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsCX->show ();
	d_vbList.pack_start (d_hbCX, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hbCX.show ();

	d_hbCY.pack_start (*d_lblCY, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblCY->show ();
	d_hsCY->set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsCY->set_digits (2);
	d_hsCY->set_draw_value (TRUE);
	d_hbCY.pack_end (*d_hsCY, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsCY->show ();
	d_vbList.pack_start (d_hbCY, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hbCY.show ();

	d_hbMinDist.pack_start (*d_lblMinDist, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblMinDist->show ();
	d_hsMinDist->set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsMinDist->set_digits (2);
	d_hsMinDist->set_draw_value (TRUE);
	d_hbMinDist.pack_end (*d_hsMinDist, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsMinDist->show ();
	d_vbList.pack_start (d_hbMinDist, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hbMinDist.show ();

	d_hbMaxDist.pack_start (*d_lblMaxDist, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblMaxDist->show ();
	d_hsMaxDist->set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsMaxDist->set_digits (2);
	d_hsMaxDist->set_draw_value (TRUE);
	d_hbMaxDist.pack_end (*d_hsMaxDist, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsMaxDist->show ();
	d_vbList.pack_start (d_hbMaxDist, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hbMaxDist.show ();

	d_vbList.pack_start (d_hSep);
	d_hSep.show ();

	d_hbScaleFactor.pack_start (*d_lblScaleFactor, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblScaleFactor->show ();
	d_hsScaleFactor->set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsScaleFactor->set_digits (2);
	d_hsScaleFactor->set_draw_value (TRUE);
	d_hbScaleFactor.pack_end (*d_hsScaleFactor, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsScaleFactor->show ();
	d_vbList.pack_start (d_hbScaleFactor, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hbScaleFactor.show ();

	d_hbSmoothDist.pack_start (*d_lblSmoothDist, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblSmoothDist->show ();
	d_hsSmoothDist->set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsSmoothDist->set_digits (2);
	d_hsSmoothDist->set_draw_value (TRUE);
	d_hbSmoothDist.pack_end (*d_hsSmoothDist, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsSmoothDist->show ();
	d_vbList.pack_start (d_hbSmoothDist, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hbSmoothDist.show ();

	d_hbInvert.pack_start (*d_lblInvert, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblInvert->show ();
	d_hbInvert.pack_end (d_btnInvert, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_btnInvert.show ();
	d_vbList.pack_start (d_hbInvert, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hbInvert.show ();

	d_hbFullDist.pack_start (*d_lblFullDist, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblFullDist->show ();
	d_hbFullDist.pack_end (d_btnFullDist, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_btnFullDist.show ();
	d_vbList.pack_start (d_hbFullDist, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hbFullDist.show ();

	d_optionsFrame.add (d_vbList);
	this->d_table->attach (d_optionsFrame, 3, 6, 1, 7);
        d_vbList.show ();
	d_optionsFrame.show ();
        this->d_baseFrame->show ();
        this->get_vbox()->show ();
}



void TFDialogLinearScale::buildDialogWindow ()
{
	this->set_usize (450, 375);
	this->set_title (this->d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	this->insertPreview ();
	insertOptions ();
	this->fillActionArea ();

	connect_to_method (d_drawArea->button_press_event, this, 
			&TFDialogLinearScale::daButtonPressedCallback);
	
	this->show ();
}


/*
 *  buttonCallbackApply: roughen the Height Field
 */
void TFDialogLinearScale::buttonCallbackApply ()
{
	float			cx, cy, sf, mind, maxd, smoothf;
	bool			invert, fulldist;

	cx = d_adjCX->get_value ();
	cy = d_adjCY->get_value ();
        sf = d_adjScaleFactor->get_value ();
        mind = d_adjMinDist->get_value (); 
	maxd = d_adjMaxDist->get_value ();
	smoothf = d_adjSmoothDist->get_value ();
	invert = d_btnInvert.get_active ();
	fulldist = d_btnFullDist.get_active ();

	if (invert)
		sf = 1-(sf-1);

	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogLinearScale::buttonCallbackApply");
	d_HFO->linearScale (cx, cy, sf, mind, maxd, smoothf, invert, fulldist);
	d_HFD->draw ();

	// only do this if window stays open 
	if (d_applyHit)
		{
		this->setHFobjs (d_HF, d_HFD);
		updatePreviewCallback ();
		}

	this->d_applyHit = TRUE;
}


/*
 *  updatePreviewCallback: update the preview after a slider has been updated.
 */
void TFDialogLinearScale::updatePreviewCallback ()
{
	if (!d_cbUsePreview.get_active())
		return;

	float		cx, cy, sf, mind, maxd, smoothf;
	bool		invert, fulldist;

	cx = d_adjCX->get_value ();
	cy = d_adjCY->get_value ();
        sf = d_adjScaleFactor->get_value ();
        mind = d_adjMinDist->get_value (); 
	maxd = d_adjMaxDist->get_value ();
	smoothf = d_adjSmoothDist->get_value ();
	invert = d_btnInvert.get_active ();
	fulldist = d_btnFullDist.get_active ();

	if (invert)
		sf = 1-(sf-1);

	SanityCheck::bailout ((!d_HFPreview), "d_HFPreview==NULL", "TFDialogLinearScale::updatePreviewCallback");
	SanityCheck::bailout ((!d_HFOPreview), "d_HFOPreview==NULL", "TFDialogLinearScale::updatePreviewCallback");
	SanityCheck::bailout ((!d_HFDPreview), "d_HFOPreview==NULL", "TFDialogLinearScale::updatePreviewCallback");
	this->previewUpToDate ();
	d_HFPreview->restore ();
	d_HFOPreview->linearScale (cx, cy, sf, mind, maxd, smoothf, invert, fulldist);
	d_HFDPreview->setColormap (d_HFD->getColormap());
	d_HFDPreview->draw ();
	d_cx = (int)(cx*(float)d_drawArea->width()); 
	d_cy = (int)(cy*(float)d_drawArea->height()); 
	this->drawCrosshairs (d_cx, d_cy);
}


/*
 *  daButtonPressedCallback: update the preview d_drawArea and adjustment 
 * 	the appropriate widgets (done when the user clicks on the preview)
 */
int TFDialogLinearScale::daButtonPressedCallback (GdkEventButton *e)
{
	if (!d_cbUsePreview.get_active())
		return 1;

	int	w = this->d_drawArea->width(),
		h = this->d_drawArea->height();

	if (e->button == 1)
		{
		d_adjCX->set_value (e->x/w);
		d_adjCY->set_value (e->y/h);
		}
	else
	if (e->button == 2)
		{
		float	d = MathTrig::distance (d_adjCX->get_value()*w, 
				d_adjCY->get_value()*h, e->x, e->y);

		d/=((d_drawArea->width()+d_drawArea->height())/2);
		if (d > 1)
			d=1;
		d_adjMinDist->set_value (d); 
		}
	else
	if (e->button == 3)
		{
		float	d = MathTrig::distance (d_adjCX->get_value()*w, 
				d_adjCY->get_value()*h, e->x, e->y);

		d/=((d_drawArea->width()+d_drawArea->height())/2);
		if (d > 1)
			d=1;
		d_adjMaxDist->set_value (d); 
		}
	updatePreviewCallback ();
	return 0;
}

